package net.shadowfacts.cacao.view.button

import net.shadowfacts.cacao.Window
import net.shadowfacts.cacao.geometry.Point
import net.shadowfacts.cacao.geometry.Rect
import net.shadowfacts.cacao.util.MouseButton
import org.junit.jupiter.api.Assertions.assertEquals
import org.junit.jupiter.api.Assertions.assertTrue
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import java.util.concurrent.CompletableFuture

/**
 * @author shadowfacts
 */
class EnumButtonTests {

	companion object {
		@BeforeAll
		@JvmStatic
		fun setupAll() {
			System.setProperty("cacao.drawing.disabled", "true")
		}
	}

	enum class MyEnum {
		ONE, TWO, THREE
	}

	lateinit var window: Window

	@BeforeEach
	fun setup() {
		window = Window()
	}

	@Test
	fun testHandlerCalled() {
		val called = CompletableFuture<Boolean>()
		val button = window.addView(EnumButton(MyEnum.ONE, MyEnum::name).apply {
			frame = Rect(0.0, 0.0, 25.0, 25.0)
			content.frame = bounds
			handler = {
				called.complete(true)
			}
		})

		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.LEFT))
		assertTrue(called.getNow(false))
		assertEquals(MyEnum.TWO, button.value)
	}

	@Test
	fun testCyclesValues() {
		val button = window.addView(EnumButton(MyEnum.ONE, MyEnum::name).apply {
			frame = Rect(0.0, 0.0, 25.0, 25.0)
			content.frame = bounds
		})

		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.LEFT))
		assertEquals(MyEnum.TWO, button.value)
		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.LEFT))
		assertEquals(MyEnum.THREE, button.value)
		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.LEFT))
		assertEquals(MyEnum.ONE, button.value)
	}

	@Test
	fun testCyclesValuesBackwards() {
		val button = window.addView(EnumButton(MyEnum.ONE, MyEnum::name).apply {
			frame = Rect(0.0, 0.0, 25.0, 25.0)
			content.frame = Rect(0.0, 0.0, 25.0, 25.0)
		})

		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.LEFT))
		assertEquals(MyEnum.TWO, button.value)
		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.RIGHT))
		assertEquals(MyEnum.ONE, button.value)
	}

	@Test
	fun testMiddleClickDoesNotChangeValue() {
		val button = window.addView(EnumButton(MyEnum.ONE, MyEnum::name).apply {
			frame = Rect(0.0, 0.0, 25.0, 25.0)
			content.frame = Rect(0.0, 0.0, 25.0, 25.0)
		})

		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.MIDDLE))
		assertEquals(MyEnum.ONE, button.value)
	}

}